## Praat script for Plotnik "button" to automatically jump to a given vowel
## written by Ingrid Rosenfelder
## last updated August 12, 2010

## updates in January 2010:
## - move cursor to time of measurement at the very end
## - perform checks to see whether sound file and TextGrid are already in the objects list (no need to open them repeatedly)
## - work with whole path names (rather than just the file names)
## - changed "name in objects list" so that dashes are NOT replaced by underscores
## - convert Mac path name format (with ":") into Unix-like format (with "/")
## - chop off hard disk name at beginning of absolute Mac path and replace with root slash
## - turn everything into procedures

## update in July 2010:
## - fixed spaces in path names of files problem

## update in August 2010:
## - rewrote procedure "name_in_objects_list" to just chop off any extension (no longer checks for correct file type)
##    to fix issue of "extension junk" generated by Plotnik when filenames get too long (???)
##    (e.g. "12345678901234567890123.TextGrid" becomes "12345678901234567890123.#B85DC8")

## input:
## name/path of sound file:  path_of_sound_file$
## name/path of corresponding TextGrid:  path_of_TextGrid$
## time of measurement: time_of_measurement
form Get input file names and time of measurement
	sentence Path_of_sound_file Howard:Users:ingrid:Documents:vowel_analysis:PlotnikButton:testbutton.wav
	sentence Path_of_TextGrid Howard:Users:ingrid:Documents:vowel_analysis:PlotnikButton:testbutton.TextGrid
	real Time_of_measurement 1234.088
endform

#clearinfo
#printline Input:  path_of_sound_file is 'path_of_sound_file$'
#printline Input:  path_of_TextGrid is 'path_of_TextGrid$'

############################ PROCEDURE DEFINITIONS###############################################

procedure convert_path_to_Unix_format .macpath$ .unixpath$
	## converts Mac path names (with ":" as path separator) to Unix conventions (with "/" as path separator)
	## and replaces the name of the hard disk in an absolute Mac path (e.g. "Max:") with a Unix root slash ("/")
	## NOTE:  this only works for absolute paths; otherwise the first directory in the path will be chopped off!
	.index_first_colon = index(.macpath$, ":")
	'.unixpath$' = "/" + right$(replace$(.macpath$, ":", "/", 0), length(.macpath$) - .index_first_colon)
endproc

procedure name_in_objects_list .path$ .name$ .type$
	## converts absolute path name of file to be opened into name of that file in the objects list
	## 1) chop off all path prefixes
	last_slash = rindex(.path$, "/")
	.name1$ = right$(.path$, length(.path$) - last_slash)

## OLD APPROACH:  check that file has correct extension for given type
#	if '.type$' = "LongSound" or '.type$' = "Sound"
#		'.name$' = replace_regex$(.name1$ - ".wav", "\W-", "_", 0)
#	elsif '.type$' = "TextGrid"
#		'.name$' = replace_regex$(.name1$ - ".TextGrid", "\W-", "_", 0)
#	else
#		pause Unknown file type!  ('.type')
#	endif

	## 2) chop off the file extension
	## NEW APPROACH:  just delete everything after the last ".":
	last_dot = rindex(.name1$, ".")
	#printline 'tab$'Index of last dot is 'last_dot'
	if last_dot <> 0
		name2$ = left$(.name1$, last_dot - 1)
		#printline 'tab$'Name without dot is 'name2$'
	else
		name2$ = .name1$
		#printline 'tab$'No dot in filename!
	endif
	## 3) convert special characters in filename
	## \W = not a word
	## (everything that is not [a-zA-Z0-9_] or "-" (single hyphen/minus) is replaced by underscores)
	#'.name3$' = replace_regex$(.name2$, "#", "_", 0)
	#'.name$' = replace_regex$(.name2$, "[^-|\W]", "_", 0)
	blubb$ = replace_regex$(name2$, "[^-|\W]", "_", 0)
	#printline Blubb is 'blubb$'
	'.name$' = blubb$
endproc

procedure in_objects_list .name$ .isopen$ .type$
	## checks whether a given file of a certain type is already in the objects list
	## (type can be "LongSound", "Sound", "TextGrid" etc.)
	select all
	for n from 1 to numberOfSelected('.type$')
		if extractWord$(selected$('.type$', n), "") = .name$
			'.isopen$' = "1"
		else
			'.isopen$' = "0"
		endif
	endfor
endproc

############################ MAIN PROGRAM STARTS HERE...##########################################

## convert Mac paths to Unix format
	## -> unix_path_of_sound_file$
	## -> unix_path_of_TextGrid$
call convert_path_to_Unix_format "'path_of_sound_file$'" unix_path_of_sound_file$
call convert_path_to_Unix_format "'path_of_TextGrid$'" unix_path_of_TextGrid$

#clearinfo 
#printline 
#printline
#printline Unix path of file 'path_of_sound_file$' is 'unix_path_of_sound_file$'
#printline Unix path of file 'path_of_TextGrid$' is 'unix_path_of_TextGrid$'

## get file names in objects list
	## -> name_of_sound_file_in_objects_list$
	## -> name_of_TextGrid_in_objects_list$
call name_in_objects_list "'unix_path_of_sound_file$'" name_of_sound_file_in_objects_list$ "LongSound"
call name_in_objects_list "'unix_path_of_TextGrid$'" name_of_TextGrid_in_objects_list$ "TextGrid"

#printline Name of sound file in objects list is 'name_of_sound_file_in_objects_list$'
#printline Name of TextGrid in objects list is 'name_of_TextGrid_in_objects_list$'

## check whether sound file and TextGrid are already in the object list
	## -> isopen_sound
	## -> isopen_TextGrid
isopen_sound$ = "0"
isopen_TextGrid$ = "0"
call in_objects_list 'name_of_sound_file_in_objects_list$' isopen_sound$ "LongSound"
call in_objects_list 'name_of_TextGrid_in_objects_list$' isopen_TextGrid$ "TextGrid"

#printline Sound file already in objects list?  ("0" = "no", "1" = "yes") 'isopen_sound$'
#printline TextGrid already in objects list?  ("0" = "no", "1" = "yes") 'isopen_TextGrid$'

## open sound file and corresponding TextGrid, if not in objects list already
if isopen_sound$ = "0"
	Open long sound file... 'unix_path_of_sound_file$'
	#printline Opened sound file.
endif
if isopen_TextGrid$ = "0"
	Read from file... 'unix_path_of_TextGrid$'
	#printline Opened TextGrid.
endif

## get number of tiers
##select TextGrid 'name_of_TextGrid_in_objects_list$'
##n_tiers = Get number of tiers

## open editor
select LongSound 'name_of_sound_file_in_objects_list$'
plus TextGrid 'name_of_TextGrid_in_objects_list$'
Edit

editor TextGrid 'name_of_TextGrid_in_objects_list$'
##	## find appropriate interval to zoom in on 
##	for i from 1 to n_tiers
##		Move cursor to... 'time_of_measurement'
##		Select interval
##		get name of interval and compare with token name

	zoom_start = time_of_measurement - 0.25
	zoom_end = time_of_measurement + 0.25

	Zoom... zoom_start zoom_end

	## move cursor to point of measurement
	Move cursor to... 'time_of_measurement'

	## Play window
	Play window

